import numpy as np
import doas.scd.fitting
import doas.scd.omno2.equation


def fit_spectra_OMNO2(wl: np.ndarray, rad: np.ndarray, eqn: doas.scd.omno2.equation.OMNO2, **kwargs):
    if eqn.consider_shifted_wavlen:
        # do the fit once without the wavelength shift

        # make a copy of the curve_fit parameters that will change when the shifting is removed
        p0 = kwargs.get('p0', None)
        bounds = kwargs.get('bounds', None)

        # set the new parameters
        if p0 is not None:
            p0_dict = eqn.parse_fit_params(*p0)
            p0_dict.pop('shifted_wavlen')
            kwargs['p0'] = eqn.unparse_fit_params(p0_dict)
        if bounds is not None:
            lower_bounds_dict = eqn.parse_fit_params(*bounds[0])
            upper_bounds_dict = eqn.parse_fit_params(*bounds[1])
            lower_bounds_dict.pop('shifted_wavlen')
            upper_bounds_dict.pop('shifted_wavlen')
            kwargs['bounds'] = (eqn.unparse_fit_params(lower_bounds_dict),
                                eqn.unparse_fit_params(upper_bounds_dict))

        # do the first fit
        eqn.consider_shifted_wavlen = False
        result = doas.scd.fitting.fit_spectra(wl, rad, eqn, **kwargs)
        if isinstance(result, list):
            result = result[0]

        # extract p0 for the 2nd iteration and restore parameters
        p0_new = eqn.unparse_fit_params(result)
        p0_new += [p0_dict['shifted_wavlen'][0]] if p0 is not None else [0.0]
        kwargs['p0'] = p0_new
        if bounds is not None:
            kwargs['bounds'] = bounds
        eqn.consider_shifted_wavlen = True

    # do the 2nd iteration with the last result as the initial guess
    return doas.scd.fitting.fit_spectra(wl, rad, eqn, **kwargs)
